@@ -19,263 +19,18 @@
  * @author Haruki Setoyama <haruki@planewave.org>
  * @copyright Copyright &copy; 2003-2004, Haruki SETOYAMA <haruki@planewave.org>
  * @license http://www.opensource.org/licenses/lgpl-license.php LGPL
- * @version 0.5- $Id: phpoot.parser.php,v 1.18 2004/04/27 02:34:00 haruki Exp $
+ * @version 0.6- $Id: phpoot.parser.php,v 1.22 2004/05/12 10:52:01 haruki Exp $
  * @link http://phpoot.sourceforge.jp/
  * @package phpoot
  * @subpackage _parser
  */
-/**
-* Requires PEAR
-*/
-require_once 'PEAR.php';
-/**
-* XML_HTMLSax Package is Required
-*/
-require_once 'XML/XML_HTMLSax.php';
-
 /**
 * attribute name for Variable Tag
 */
 @define('PHPOOT_VAR', 'var');
-
-/**
-* handler for HTMLSax
-* parse of template is done with phpoot_parser classes actually
-* @package phpoot
-* @subpackage _parser
-*/
-class phpoot_handler {
-
-    /**
-     * option
-     * <pre>
-     * 'escape''remove'     when true, remove all the <! ... >
-     * 'pi''remove'         when true, remove all the <? ... ?>
-     * 'jasp''remove'       when true, remove all the <% ... %>
-     * 'tag''empty'         names of empty tag which should be < /> style
-     * 'tag''raw'           extention of model data with HTML.
-     * 'tag''allow_no_brace'    variable name is normaly "{foo}" style. when true, allow just "foo"
-     * </pre>
-     * @var array
-     */
-    var $option;
-
-    /**
-     * template text
-     * @var string
-     * @access private
-     */
-    var $_template = '';
-
-    /**
-     * blanck phpoot_parser_tag object
-     * @var array of phpoot_parser_tag
-     * @access private
-     */
-    var $_tag_parsers = array();
-    /**
-     * @access private
-     */
-    var $_level = 0;
-    /**
-     * @access private
-     */
-    var $_stack = array();
-
-    /**
-     * constructor
-     * @access public
-     */
-    function phpoot_handler(&$option, $parsers = array('num', 'variable'))
-    {
-        // option
-        $default = array(
-            'tag' => array(
-                'empty' => array(
-                    'br' => 1,
-                    'param' => 1,
-                    'hr' => 1,
-                    'input' => 1,
-                    'col' => 1,
-                    'img' => 1,
-                    'area' => 1,
-                    'frame' => 1,
-                    'meta' => 1,
-                    'link' => 1,
-                    'base' => 1,
-                    'basefont' => 1,
-                    'isindex' => 1
-                ),
-                'raw' => array(),
-                'allow_no_brace' => false
-            ),
-            'data' => array(),
-            'escape' => array(
-                'remove' => false
-            ),
-            'pi' => array(
-                'remove' => true
-            ),
-            'jasp' => array(
-                'remove' => true
-            ),
-            '_merged' => 1
-        );
-        $this->option =& $option;
-        if (! isset($this->option['_merged'])) {
-            $this->option = array_merge_recursive($default, $this->option);
-        }
-    
-        $this->_tag_parsers = array();
-        foreach ($parsers as $parser) {
-            $class = 'phpoot_parser_tag_' . $parser;
-            $this->_tag_parsers[] = new $class();
-        }
-        $this->_tag_parsers[] = new phpoot_parser_tag_static();
-    }
-
-    /**
-     * initialize before HTMLSax parse
-     * @access public
-     */
-    function init(&$template)
-    {
-        // template
-        $template = str_replace(array("\r\n", "\r"), array("\n", "\n"), $template);
-        $this->_template =& $template;
-        
-        // init private prop.
-        $this->_level = 0;
-        $this->_error = array();
-        $this->_stack = array();
-        $this->_stack[0] = new phpoot_parser_root;
-        
-        // option
-        $num = count($this->_tag_parsers);
-        for ($i=0; $i<$num; $i++) {
-            $this->_tag_parsers[$i]->setOption($this->option['tag']);
-        } 
-    }
-
-    /**
-     * set handler on HTMLSax parser
-     * @access public
-     */
-    function configParser(&$parser)
-    {
-        $parser->set_element_handler('openHandler', 'closeHandler');
-        $parser->set_data_handler('dataHandler');
-        $parser->set_escape_handler('escapeHandler');
-        $parser->set_pi_handler('piHandler');
-        $parser->set_jasp_handler('jaspHandler');
-        $parser->set_option('XML_OPTION_FULL_ESCAPES');
-    }
-
-    /**
-     * get parsed template, that is PHP script
-     * @access public
-     */
-    function getParsed()
-    {
-        while($this->_level > 0) {
-            $this->_stack[$this->_level]->close(strlen($this->_template));
-            $this->_level--;
-        }
-
-        return $this->_stack[$this->_level]->parse();
-    }
-
-    /**
-     * handler method for tag open
-     * @access public
-     */
-    function openHandler(&$parser, $name, $attrs)
-    {
-        $name = strtolower($name);
-        $attrs = array_change_key_case($attrs, CASE_LOWER);
-        $this->open_position = $parser->get_current_position();
-
-        $i=0;
-        while($this->_tag_parsers[$i]->accept($name, $attrs) === false){
-            $i++;
-        }
-
-        $this->_level++;
-        unset($this->_stack[$this->_level]);
-        $this->_stack[$this->_level] = $this->_tag_parsers[$i]; // copy
-
-        $this->_stack[$this->_level]->open(
-            $parser->get_current_position()
-            , $name
-            , $attrs
-            , $this->_stack[$this->_level -1]->info
-        );
-
-        $this->_stack[$this->_level -1]->data[] =& $this->_stack[$this->_level];
-    }
-
-    /**
-     * handler method for tag close
-     * @access public
-     */
-    function closeHandler(&$parser, $close_tag)
-    {
-        $close_tag = strtolower($close_tag);
-
-        $this->_stack[$this->_level]->close($parser->get_current_position());
-
-        if ($this->_level > 0) {
-            $this->_level--;
-            if ($this->_stack[$this->_level +1]->open_tag() != $close_tag) {
-                $this->closeHandler($parser, $close_tag);
-            }
-        }
-    }
-
-    /**
-     * handler method for data
-     * @access public
-     */
-    function dataHandler(&$parser, $data)
-    {
-        $this->_stack[$this->_level]->data[]
-            = new phpoot_parser_data($data, $this->option['data']);
-    }
-
-    /**
-     * handler method for escape
-     * @access public
-     */
-    function escapeHandler(&$parser, $data)
-    {
-        $this->_stack[$this->_level]->data[]
-            = new phpoot_parser_escape($data, $this->option['escape']);
-    }
-
-    /**
-     * handler method for pi
-     * @access public
-     */
-    function piHandler(&$parser, $target, $data)
-    {
-        $this->_stack[$this->_level]->data[]
-            = new phpoot_parser_pi($target, $data, $this->option['pi']);
-    }
-
-    /**
-     * handler method for jasp
-     * @access public
-     */
-    function jaspHandler(&$parser, $data)
-    {
-        $this->_stack[$this->_level]->data[]
-            = new phpoot_parser_jasp($data, $this->option['jasp']);
-    }
-}
-
 /**
 * base class of template parser classes
-* This class and its sub classes are accessed only by phpoot_handler class
+* This class and its sub classes are accessed only by phpoot_handler class.
 * This class is also used as null object.
 * @see phpoot_handler
 * @package phpoot
@@ -295,12 +50,14 @@ class phpoot_parser {
     var $linefeed = '';
 
     /**
+     * content data
      * @access private
+     * @var string
      */
-    var $_data = '';
+    var $_content = '';
 
     /**
-     * parse into PHP script
+     * parse data and compile it into PHP script
      * @return string
      */
     function parse()
@@ -319,7 +76,7 @@ class phpoot_parser {
 
     /**
      * returns indent for next contents
-     * @return string
+     * @return string|null
      */
     function indent_for_next()
     {
@@ -327,8 +84,8 @@ class phpoot_parser {
     }
 
     /**
-     * returns linefeed "\n" for previous contents
-     * @return string
+     * returns linefeed "\n" for previous content
+     * @return string|null
      */
     function linefeed_for_prev()
     {
@@ -336,7 +93,25 @@ class phpoot_parser {
     }
 
     /**
-     * returns if this content is empty ''
+     * returns indent for close tag
+     * @return string|null
+     */
+    function indent_for_tag()
+    {
+        return '';
+    }
+
+    /**
+     * returns linefeed "\n" for open tag
+     * @return string|null
+     */
+    function linefeed_for_tag()
+    {
+        return '';
+    }
+
+    /**
+     * returns if it is empty
      * @return string
      */
     function is_empty()
@@ -352,44 +127,90 @@ class phpoot_parser {
     {
         return false;
     }
+
+    /**
+     * returns if this content represent format to display
+     * @return array|false
+     */
+    function formatable()
+    {
+        return false;
+    }
 }
 
+////////////////////////////////////////
+
 /**
 * parser for data
 * @package phpoot
 */
 class phpoot_parser_data extends phpoot_parser {
 
-    function phpoot_parser_data($data, $option) {
-        $this->_data = $data;
+    function phpoot_parser_data($data)
+    {
+        $this->_content = $data;
     }
 
-    function parse() {
-        return $this->_data;
+    function parse()
+    {
+        return $this->_content;
     }
 
-    function indent_for_next() {
-        if (preg_match('/^(.*\n)([\040\011]*)$/sD', $this->_data, $match)) {
-            $this->_data = $match[1];
+    function indent_for_next()
+    {
+        if (preg_match('/^(.*\n)([\040\011]*)$/sD', $this->_content, $match)) {
+            $this->_content = $match[1];
             return $match[2];
         } else {
             return '';
         }
     }
 
-    function linefeed_for_prev() {
-        if (substr($this->_data, 0, 1) == "\n") {
-            $this->_data = substr($this->_data, 1);
-            return "\n";
+    function linefeed_for_prev()
+    {
+        if (preg_match('/^([\040\011]*\n)/sD', $this->_content, $match)) {
+            $this->_content = substr($this->_content, strlen($match[1]));
+            return $match[1];
+        } else {
+            return '';
+        }
+    }
+/*
+    function indent_for_tag()
+    {
+        if (preg_match('/^(.*)(\n[\040\011]*)$/sD', $this->_content, $match)) {
+            $this->_content = $match[1];
+            return $match[2];
         } else {
             return '';
         }
     }
 
-    function is_empty() {
-        return ($this->_data == '');
+    function linefeed_for_tag()
+    {
+        if (preg_match('/^([\040\011]*\n[\040\011]*)/sD', $this->_content, $match)) {
+            $this->_content = substr($this->_content, strlen($match[1]));
+            return $match[1];
+        } else {
+            return '';
+        }
+    }
+    */
+
+    function is_empty()
+    {
+        return ($this->_content == '');
     }
 
+    function formatable()
+    {
+        $data = trim($this->_content);
+        if (preg_match('/^\{([a-z_]+)\s+"([^"]*)"\}$/D', $data, $match)
+        && file_exists(PHPOOT_FORMAT . $match[1]. '.php')) {
+            return array($match[1], $match[2]);
+        }
+        return false;
+    }
 }
 
 /**
@@ -397,31 +218,15 @@ class phpoot_parser_data extends phpoot_parser {
 * @package phpoot
 */
 class phpoot_parser_escape extends phpoot_parser {
-    /**
-     * @access private
-     */
-    var $_remove = false;
 
-    function phpoot_parser_escape($data, $option) {
-        $this->_data = $data;
-        $this->_remove = (bool)$option['remove'];
+    function phpoot_parser_escape($data)
+    {
+        $this->_content = $data;
     }
 
-    function parse() {
-        if (! $this->_remove) {
-            return '<!' . $this->_data . '>';
-        } else {
-            return
-                  "<?php if (\$_error) { ?>\n"
-                . $this->indent
-                . '<!-- escape removed -->'
-                . $this->linefeed
-                . "<?php } ?>\n";;
-        }
-    }
-
-    function need_indent() {
-        return $this->_remove;
+    function parse()
+    {
+        return '<!' . $this->_content . '>';
     }
 }
 
@@ -430,36 +235,18 @@ class phpoot_parser_escape extends phpoot_parser {
 * @package phpoot
 */
 class phpoot_parser_pi extends phpoot_parser {
-    /**
-     * @access private
-     */
-    var $_remove = false;
-    /**
-     * @access private
-     */
+
     var $_target;
 
-    function phpoot_parser_pi($target, $data, $option) {
+    function phpoot_parser_pi($target,$data)
+    {
+        $this->_content = $data;
         $this->_target = $target;
-        $this->_data = $data;
-        $this->_remove = (bool)$option['remove'];
     }
 
-    function parse() {
-        if (! $this->_remove) {
-            return '<?' . $this->_target . ' ' . $this->_data . '?>';
-        } else {
-            return
-                 "<?php if (\$_error) { ?>\n"
-                . $this->indent
-                . '<!-- pi removed -->'
-                . $this->linefeed
-                . "<?php } ?>\n";
-        }
-    }
-
-    function need_indent() {
-        return $this->_remove;
+    function parse()
+    {
+        return '<?' . $this->_target . ' ' . $this->_content . '?>';
     }
 }
 
@@ -468,34 +255,45 @@ class phpoot_parser_pi extends phpoot_parser {
 * @package phpoot
 */
 class phpoot_parser_jasp extends phpoot_parser {
-    /**
-     * @access private
-     */
-    var $_remove = false;
 
-    function phpoot_parser_jasp($data, $option) {
-        $this->_data = $data;
-        $this->_remove = (bool)$option['remove'];
+    function phpoot_parser_jasp($data)
+    {
+        $this->_content = $data;
     }
 
-    function parse() {
-        if (! $this->_remove) {
-            return '<%' . $this->_data . '%>';
-        } else {
-            return
-                  "<?php if (\$_error) { ?>\n"
-                . $this->indent
-                . '<!-- jasp removed -->'
-                . $this->linefeed
-                . "<?php } ?>\n";
-        }
-    }
 
-    function need_indent() {
-        return $this->_remove;
+    function parse()
+    {
+        return '<%' . $this->_content . '%>';
     }
 }
 
+/**
+* parser for removed data
+* @package phpoot
+*/
+class phpoot_parser_remove extends phpoot_parser {
+
+    function need_indent()
+    {
+        return true;
+    }
+}
+
+/**
+* parser for empty data
+* @package phpoot
+*/
+class phpoot_parser_empty extends phpoot_parser_remove {
+
+    function is_empty()
+    {
+        return true;
+    }
+}
+
+////////////////////////////////////////
+
 /**
  * Base of parser for tag
  * @package phpoot
@@ -503,8 +301,8 @@ class phpoot_parser_jasp extends phpoot_parser {
  */
 class phpoot_parser_tag extends phpoot_parser {
     /**
-     * data in the tag
-     * @var phpoot_parser
+     * data in this tag
+     * @var array of phpoot_parser
      */
     var $data = '';
 
@@ -514,56 +312,60 @@ class phpoot_parser_tag extends phpoot_parser {
      */
     var $info;
 
+    /**
+     * options
+     * @var array
+     */
+    var $option;
+
+    /**
+     * common information in phpoot_parser_tag of all level
+     */
+    var $common;
+
     /**
      * name of open tag
      * @access private
      */
     var $_open_tag;
-
     /**
      * attribute in the open tag
      * @var array
      * @access private
      */
     var $_attrs;
-
-    /**
-     * options
-     * @var array
-     * @access private
-     */
-    var $_option;
-
     /**
      * if this tag can have data or not
      * @var bool
      * @access private
      */
     var $_empty_tag = false;
-
     /**
      * position when tag is opend
      * @access private
      */
     var $_position;
-
     /**
      * tags that will be deleted when no attribute
      * @var array
      */
     var $_delete = array('span' => 1);
-
     /**
-     * set option
-     * @access public
+     * if content is to be formated or not
+     * @access private
      */
-    function setOption($option)
-    {
-        $this->_option = $option;
-    }
+    var $_format = null;
+    /**
+     * @access private
+     */
+    var $_linefeed_open_tag = '', $_indent_close_tag = '';
+    /**
+     * @access private
+     */
+    var $_closed = false;
 
     /**
-     * judge if this class accept the tag
+     * judges if this class accept the tag
      * @abstract
      * @access public
      * @return bool
@@ -580,71 +382,87 @@ class phpoot_parser_tag extends phpoot_parser {
     function open($position, $name, $attrs, &$info) {}
 
     /**
-     * initialize
+     * common initializations
      * @access private
      */
-    function _open($position, $name, $attrs) {
+    function _open($position, $name, $attrs)
+    {
         $this->_position = $position;
         $this->_open_tag = $name;
         $this->_attrs = $attrs;
 
-        if (isset($option['empty'][$name])) {
+        if (isset($this->option['empty'][$name])) {
             $this->_empty_tag = true;
         }
 
-        $this->_init();
-    }
-
-    /**
-     * initialize data stack
-     * @access private
-     */
-    function _init() {
         $this->data = array();
-        $this->data[] = new phpoot_parser;
+        $this->data[] = new phpoot_parser; // null obj
     }
 
     /**
-     * parse all the data in $this->data stack and add the results on $this->_data
+     * parse all the data in $this->data stack
+     * and set its compiled PHP script into $this->_content.
+     * $this->_format
      * @access public
      */
-    function close($position) {
+    function close($position)
+    {
+        if ($this->_closed) {
+            exit('close again!'); // for debug.
+        }
+
         if ($this->_position == $position) {
             $this->_empty_tag = true;
         }
 
-        $this->data[] = new phpoot_parser;
-        $num = count($this->data) -1;
-
-        // get indent and linefeed for pritty print
-        for ($i = $num-1; $i > 0; $i--) {
-            if ($this->data[$i]->need_indent()) {
-                $this->data[$i]->indent = $this->data[$i -1]->indent_for_next();
-                $this->data[$i]->linefeed = $this->data[$i +1]->linefeed_for_prev();
+        switch(count($this->data)){
+        case 1:
+            $this->_content = '';
+            break;
+        case 2:
+            if (($format = $this->data[1]->formatable()) != false) {
+                $this->_format = $format;
             }
-        }
+            $this->_indent_close_tag = $this->data[1]->indent_for_next();
+            $this->_content = $this->data[1]->parse();
+            break;
+        default:
+            $this->data[] = new phpoot_parser;
+            $num = count($this->data) -1;
 
-        // altarnative & parse
-        for ($i=1; $i < $num; $i++) {
-            if ($this->data[$i]->is_alternative() == true
-            && $this->data[$i+1]->is_empty()) {
-                $alts = $this->_get_alternative($i, $num, $this->data[$i]);
-                if ($alts > 0) {
-                    $this->_data .= $this->_parse_alternative($i, $alts);
-                    $i += $alts*2;
-                    continue;
+            // get indent and linefeed for pritty print
+            $this->_indent_close_tag = $this->data[$num-1]->indent_for_next();
+            for ($i = $num-1; $i > 0; $i--) {
+                if ($this->data[$i]->need_indent()) {
+                    $this->data[$i]->indent = $this->data[$i -1]->indent_for_next();
+                    $this->data[$i]->linefeed = $this->data[$i +1]->linefeed_for_prev();
                 }
             }
-            $this->_data .= $this->data[$i]->parse();
-        }
+            $this->_linefeed_open_tag = $this->data[1]->linefeed_for_prev();
 
-        $this->_init();
+            // altarnative & parse
+            $this->_content = '';
+            for ($i=1; $i < $num; $i++) {
+                if ($this->data[$i]->is_alternative() == true
+                && $this->data[$i+1]->is_empty()) {
+                    $alts = $this->_get_alternative($i, $num, $this->data[$i]);
+                    if ($alts > 0) {
+                        $this->_content .= $this->_parse_alternative($i, $alts);
+                        $i += $alts*2;
+                        continue;
+                    }
+                }
+                $this->_content .= $this->data[$i]->parse();
+            }
+
+            $this->_closed = true; // for debug.
+        }
     }
 
     /**
      * get alternative tags
      * @access private
-     * @return int  number of the alternatives
+     * @return int  the number of alternatives
      */
     function _get_alternative($i, $num, &$phpoot_parser) {
         $open_tag = $phpoot_parser->open_tag();
@@ -711,6 +529,15 @@ class phpoot_parser_tag extends phpoot_parser {
         return $this->_open_tag;
     }
 
+    /**
+     * returns position of open tag
+     * @return int
+     */
+    function open_position()
+    {
+        return $this->_position;
+    }
+
     /**
      * parse tag and set $this->_parsed_close_tag and $this->_parsed_open_tag
      * @access private
@@ -759,24 +586,46 @@ class phpoot_parser_tag extends phpoot_parser {
         // deletable tag?
         if ($attr_statics == 0 && isset($this->_delete[$this->_open_tag])) {
             if (empty($attr_variables)) {
-                $open_tag  = '';
-                $close_tag = '';
+                $open_tag = (strlen($this->_linefeed_open_tag) > 0)
+                    ? ''
+                    : $this->indent;
             } else {
                 $_condition = implode(" != '' || ", $attr_variables) . " != ''";
-                $open_tag = "<?php if ($_condition) { ?>\n" . $open_tag . "<?php } ?>\n";
-                if ($close_tag != '') {
-                    $close_tag = "<?php if ($_condition) { ?>\n" . $close_tag . "<?php } ?>\n";
+                if (strlen($this->_linefeed_open_tag) > 0) {
+                    $open_tag = "<?php if ($_condition) { ?>\n" . $this->indent
+                        . $open_tag . $this->_linefeed_open_tag . "<?php } ?>\n";
+                } else {
+                    $open_tag = $this->indent . "<?php if ($_condition) { ?>\n"
+                        . $open_tag . "<?php } ?>\n";
+                }
+            }
+            if (empty($attr_variables) || $close_tag == '') {
+                $close_tag = (strlen($this->_indent_close_tag) > 0)
+                    ? ''
+                    : $this->linefeed;
+            } else {
+                if (strlen($$this->_indent_close_tag) > 0) {
+                    $close_tag = "<?php if ($_condition) { ?>\n"
+                        . $this->_indent_close_tag . $close_tag . $this->linefeed . "<?php } ?>\n";
+                } else {
+                    $close_tag = "<?php if ($_condition) { ?>\n"
+                        . $close_tag . "<?php } ?>\n" . $this->linefeed;
                 }
             }
         }
+        else {
+            $open_tag = $this->indent . $open_tag . $this->_linefeed_open_tag;
+            $close_tag = $this->_indent_close_tag . $close_tag . $this->linefeed;
+        }
 
-        return array($this->indent . $open_tag, $close_tag . $this->linefeed);
+        return array($open_tag, $close_tag);
     }
 
     /**
      * get indent for close tag
      * @access private
      * @return string
+     * @see _parse_tag()
      */
     function _indent_close_tag() {
         // for static tag
@@ -793,6 +642,16 @@ class phpoot_parser_tag extends phpoot_parser {
         return '"' . str_replace(array('"', '<', '>'), array('&quot', '&lt;', '&gt;'), $str) . '"';
     }
 
+    /**
+     * single quote for php
+     * @access private
+     * @return string
+     */
+    function _single_quote($str)
+    {
+        return '\'' . str_replace('\'', '\\\'', $str) . '\'';
+    }
+
 }
 
 /**
@@ -800,24 +659,34 @@ class phpoot_parser_tag extends phpoot_parser {
  *
  * @package phpoot
  */
-class phpoot_parser_root extends phpoot_parser_tag {
+class phpoot_parser_tag_root extends phpoot_parser_tag {
 
+    /**
+     * information of variable tag in each level
+     */
     var $info = array('level' => 0, 'nests' => false);
 
-    function phpoot_parser_root() {
-        $this->_init();
+    function phpoot_parser_tag_root()
+    {
+        $this->data = array();
+        $this->data[] = new phpoot_parser; // null obj
     }
 
     function close($position)
     {
-        // todo: error
+        // do nothing. Infact, this is not called.
         return;
     }
 
     function parse()
     {
         parent::close(0);
-        return $this->_data;
+        foreach($this->common['format'] as $type) {
+            $this->_content =
+                "<?php require_once PHPOOT_FORMAT.'$type'.'.php'; ?>\n"
+                . $this->_content;
+        }
+        return $this->_content;
     }
 }
 
@@ -843,7 +712,7 @@ class phpoot_parser_tag_static extends phpoot_parser_tag {
     function parse()
     {
         list($opentag, $closetag) = $this->_parse_tag();
-        return $opentag . $this->_data . $closetag;
+        return $opentag . $this->_content . $closetag;
     }
 }
 
@@ -854,17 +723,16 @@ class phpoot_parser_tag_static extends phpoot_parser_tag {
  * @package phpoot
  */
 class phpoot_parser_tag_variable extends phpoot_parser_tag {
+   /**
+     * flag for alternation
+     * @access private
+     */
+    var $_alt = 0;
 
     function accept($open_tag, $attrs)
     {
-        if (! isset($attrs[PHPOOT_VAR])) {
-            return false;
-        }
-        if (preg_match('/^\{[a-zA-Z_\x7f-\xff][a-zA-Z0-9_\x7f-\xff]*\}$/D', $attrs[PHPOOT_VAR])) {
-            return true;
-        }
-        if ($this->_option['allow_no_brace'] == true
-        && preg_match('/^[a-zA-Z_\x7f-\xff][a-zA-Z0-9_\x7f-\xff]*$/D', $attrs[PHPOOT_VAR])) {
+        if (isset($attrs[PHPOOT_VAR])
+        && preg_match('/^\{[a-zA-Z_\x7f-\xff][a-zA-Z0-9_\x7f-\xff]*\}\+?$/D', $attrs[PHPOOT_VAR])) {
             return true;
         }
         return false;
@@ -875,10 +743,9 @@ class phpoot_parser_tag_variable extends phpoot_parser_tag {
         $this->info['level'] = (int)$info['level'] +1;
         $this->info['nests'] = false;
         $info['nests'] = true;
-        if (preg_match('/^\{([a-zA-Z_\x7f-\xff][a-zA-Z0-9_\x7f-\xff]*)\}$/D', $attrs[PHPOOT_VAR], $match)) {
+        if (preg_match('/^\{([a-zA-Z_\x7f-\xff][a-zA-Z0-9_\x7f-\xff]*)\}(\+?)$/D', $attrs[PHPOOT_VAR], $match)) {
             $this->info['name'] = $match[1];
-        } else {
-            $this->info['name'] = $attrs[PHPOOT_VAR];
+            $this->_alt = strlen($match[2]);
         }
 
         $this->_open($position, $name, $attrs);
@@ -915,35 +782,46 @@ class phpoot_parser_tag_variable extends phpoot_parser_tag {
 
         $parsed
             = "if ($_valcurr === null || $_valcurr === false) { "
-            .  "if (\$_error) { ?>\n"
-            .  $this->indent
-            .  "<!-- NO DATA for '$_var' -->"
-            .  $this->linefeed
-            .  "<?php } "
-            .  "} else { $_num++; ";
+            . "if (!\$_quiet) { ?>\n"
+            . $this->indent
+            . "<!-- NO DATA for '$_var' -->"
+            . $this->linefeed
+            . "<?php } "
+            . "} else { $_num++; ";
 
         if (! $this->info['nests']) {
             $parsed
                 .= "?>\n"
                 .  $open_tag;
 
-            $_func = 'phpoot_string';
-            foreach ($this->_option['raw'] as $ext) {
-                if (substr($_var, -strlen($ext)) == $ext) {
-                    $_func = 'phpoot_raw';
+            if (! isset($this->_format)) {
+            // normal data
+                $_func = 'phpoot_string';
+                foreach ($this->option['raw'] as $ext) {
+                    if (substr($_var, -strlen($ext)) == $ext) {
+                        $_func = 'phpoot_raw';
+                    }
                 }
-            }
 
-            $parsed
-                .= "<?php if ($_valcurr === true) { ?>\n"
-                .  $this->_data
-                .  "<?php } else { $_func($_valcurr, '$_var', \$_error); } ?>\n"
-                .  $close_tag;
+                $parsed
+                    .= "<?php if ($_valcurr === true) { ?>\n"
+                    .  $this->_content
+                    .  "<?php } else { $_func($_valcurr, '$_var', \$_quiet); } ?>\n"
+                    .  $close_tag;
+            } else {
+            // data to be formated
+                $_func = 'phpoot_format_' . $this->_format[0];
+                $_format = $this->_single_quote($this->_format[1]);
+                $parsed
+                    .= "<?php $_func($_valcurr, '$_var', $_format, \$_quiet); ?>\n"
+                    .  $close_tag;
+                $this->common['format'][$this->_format[0]] = $this->_format[0];
+            }
         } else {
             $parsed
                 .= "phpoot_to_array($_valcurr, '$_var'); ?>\n"
                  . $open_tag
-                 . $this->_data
+                 . $this->_content
                  . $close_tag;
         }
 
@@ -956,9 +834,9 @@ class phpoot_parser_tag_variable extends phpoot_parser_tag {
     function _indent_close_tag()
     {
         // indent before closetag
-        if (preg_match('/^(.*\n)([\040\011]*)$/sD', $this->_data, $match)
+        if (preg_match('/^(.*\n)([\040\011]*)$/sD', $this->_content, $match)
         && substr($match[1], -3) != ' ?>') {
-            $this->_data = $match[1];
+            $this->_content = $match[1];
             return $match[2];
         } else {
             return '';
@@ -972,12 +850,12 @@ class phpoot_parser_tag_variable extends phpoot_parser_tag {
 
     function is_alternative()
     {
-        return true;
+        return (bool)$this->_alt;
     }
 }
 
 /**
- * Parser for tags with '#num' id attribute
+ * Parser for tags with '#num' of attribute for variable
  *
  * @package phpoot
  */
@@ -987,10 +865,6 @@ class phpoot_parser_tag_num extends phpoot_parser_tag_static {
         if (@$attrs[PHPOOT_VAR] == '{#num}') {
                 return true;
         }
-        if ($this->_option['allow_no_brace'] == true
-        && @$attrs[PHPOOT_VAR] == '#num') {
-                return true;
-        }
         return false;
     }
 
@@ -1009,4 +883,44 @@ class phpoot_parser_tag_num extends phpoot_parser_tag_static {
     }
 }
 
+/**
+ * Parser for tags with '#num' of attribute for variable
+ *
+ * @package phpoot
+ */
+class phpoot_parser_tag_notop extends phpoot_parser_tag_static {
+    function accept($open_tag, $attrs)
+    {
+        if (@$attrs[PHPOOT_VAR] == '{#notop}') {
+                return true;
+        }
+        return false;
+    }
+
+    function open($position, $name, $attrs, &$info)
+    {
+        parent::open($position, $name, $attrs, $info);
+        unset($this->_attrs[PHPOOT_VAR]);
+    }
+
+    function parse()
+    {
+        list($open_tag, $close_tag) = $this->_parse_tag();
+
+        $_num = '$_num' . $this->info['level'];
+        $indent = (strlen($this->linefeed) > 0)
+            ? ''
+            : $this->indent;
+        return "<?php if($_num>1) { ?>\n"
+            . $open_tag . $this->_content . $close_tag
+            . "<?php } else { ?>\n"
+            . $indent
+            . "<?php } ?>\n";
+    }
+
+    function need_indent()
+    {
+        return true;
+    }
+}
 ?>
\ No newline at end of file
