@@ -19,7 +19,7 @@
  * @author Haruki Setoyama <haruki@planewave.org>
  * @copyright Copyright &copy; 2003-2004, Haruki SETOYAMA <haruki@planewave.org>
  * @license http://www.opensource.org/licenses/lgpl-license.php LGPL
- * @version 0.5 $Id: phpoot.php,v 1.33 2004/04/27 02:33:41 haruki Exp $
+ * @version 0.6 $Id: phpoot.php,v 1.38 2004/05/12 10:52:01 haruki Exp $
  * @link http://phpoot.sourceforge.jp/
  * @package phpoot
  */
@@ -37,6 +37,10 @@ define('PHPOOT', true);
  * directory where PHPOOT library is in
  */
 @define('PHPOOT_DIR', dirname(__FILE__));
+/**
+ * directory where data-formatting library is in
+ */
+@define('PHPOOT_FORMAT', PHPOOT_DIR.'/format/');
 /**
  * extention added to compiled template
  */
@@ -47,6 +51,7 @@ define('PHPOOT', true);
 define('PHPOOT_ERR_INTERNAL', 1);
 define('PHPOOT_ERR_TEMPLATE', 2);
 define('PHPOOT_ERR_COMPILE_DIR', 4);
+define('PHPOOT_WARN_TEMPLATE', 8);
 /**
  * PHPOOT Main Class.
  *
@@ -179,6 +184,7 @@ class phpoot extends PEAR {
     {
         $compiled_file = $this->_compiled_file();
         if ($this->option['force_compile'] != false
+        || $this->option['debug'] != false
         || file_exists($compiled_file) != true) {
             $err = $this->compile();
             if ($err !== true) {
@@ -193,12 +199,42 @@ class phpoot extends PEAR {
             }
         }
 
-        $err = phpoot_display_compiled($compiled_file, $data, $this->option['debug']);
+        if ($this->option['debug']
+        || $this->_default_error_mode & PEAR_ERROR_PRINT) {
+            $quiet = false;
+        } else {
+            $quiet = true;
+        }
+        $err = phpoot_display_compiled($compiled_file, $data, $quiet);
         if ($err !== true) {
+            $this->removeCompiled();
             return $this->raiseError(
                     "Failed to run the compiled file '$compiled_file'. ",
                     PHPOOT_ERR_INTERNAL);
         }
+        if ($this->option['debug'] != false) {
+            $this->removeCompiled();
+        }
+        return true;
+    }
+
+    /**
+     * displays the template results with eval() function
+     * this is for dubug porpose
+     *
+     * @param array $_val0   model data
+     * @return bool|PEARError
+     */
+    function display_eval($_val0)
+    {
+        $parsed = $this->_parse();
+        if (PEAR::isError($parsed)) return $parsed;
+        $err = @eval('?>'.$parsed);
+        if ($err !== true) {
+            return $this->raiseError(
+                    "Failed to eval compiled template. ",
+                    PHPOOT_ERR_INTERNAL);
+        }
         return true;
     }
 
@@ -255,10 +291,11 @@ class phpoot extends PEAR {
         if ($lock !== true) { return $err; }
 
         //
-        $template = $this->_get_template();
-        if (PEAR::isError($template)) return $template;
-
-        $parsed = $this->_parse($template);
+        $parsed = $this->_parse();
+        if (PEAR::isError($parsed)) {
+            $this->_unlock();
+            return $parsed;
+        }
         $parsed = "<?php if(! defined('PHPOOT')) exit(); ?>\n"
                     . $parsed . "<?php \$_check = true; ?>\n"; // <?
 
@@ -385,22 +422,35 @@ class phpoot extends PEAR {
     /**
      * parse template file
      *
-     * @param  string $template
      * @access private
      * @return string parsed PHP code
      */
-    function _parse($template)
+    function _parse()
     {
+        $template = $this->_get_template();
+        if (PEAR::isError($template)) return $template;
+
         if (!isset($this->_parser) || !isset($this->_handler)) {
-            include_once PHPOOT_DIR . '/phpoot.parser.php';
+            require_once 'XML/XML_HTMLSax.php';
+            require_once PHPOOT_DIR . '/phpoot.handler.php';
             $this->_parser = &new XML_HTMLSax();
             $this->_handler = new phpoot_handler($this->option);
+            $this->_handler->setErrorHandling(
+                    $this->_default_error_mode,
+                    $this->_default_error_options
+                );
             $this->_parser->set_object($this->_handler);
             $this->_handler->configParser($this->_parser);
         }
 
         $this->_handler->init($template);
         $this->_parser->parse($template);
+        if ($this->_handler->error > 0) {
+            $tmpl = $this->_template_file;
+            $this->raiseError(
+                    "Some errors/warnings in template file '$tmpl'. ",
+                    PHPOOT_WARN_TEMPLATE);
+        }
         return $this->_handler->getParsed();
     }
 
@@ -465,6 +515,21 @@ class phpoot extends PEAR {
         fclose ($fd);
         return $contents;
     }
+
+    /**
+     * config format
+     */
+    function setFormatOption($type, $option)
+    {
+        require_once PHPOOT_FORMAT.$type.'.php';
+        if (! function_exists('phpoot_format_'.$type.'_option')) {
+            return $this->raiseError(
+                "Failed to set format options for $type. ",
+                PHPOOT_ERR_INTERNAL);
+        }
+        call_user_func('phpoot_format_'.$type.'_option', $option);
+        return true;
+    }
 }
 
 
@@ -473,11 +538,11 @@ class phpoot extends PEAR {
  *
  * @param stirng    $compiled_file  path of compiled template file
  * @param array     $_val0          model data
- * @param bool      $_error         if this displayes error or not
+ * @param bool      $_quiet         if this displayes warning/notice or not
  * @access private
  * @return bool
  */
-function phpoot_display_compiled($compiled_file, $_val0, $_error =false)
+function phpoot_display_compiled($compiled_file, $_val0, $_quiet =true)
 {
     $_check = false; // flag for parse error check
     include $compiled_file;
@@ -514,11 +579,10 @@ function phpoot_to_array(&$scalar, $var)
 }
 
 /**
- * returns string data for display
- * @return string
+ * print string data
  * @access private
  */
-function phpoot_string($val, $var, $error)
+function phpoot_string($val, $var, $quiet =true)
 {
     if (is_string($val) || is_numeric($val)) {
         echo phpoot_htmlspecialchars($val);
@@ -528,7 +592,7 @@ function phpoot_string($val, $var, $error)
         echo phpoot_htmlspecialchars($val[$var]);
     }
     else {
-        echo ($error) ? "<!-- NON STRING DATA '$var' -->" : '';
+        echo (! $quiet) ? "<!-- NON STRING DATA '$var' -->" : '';
     }
 }
 
@@ -543,11 +607,10 @@ function phpoot_htmlspecialchars($val)
 }
 
 /**
- * returns string data for display
- * @return string
+ * print string data for display
  * @access private
  */
-function phpoot_raw($val, $var, $error)
+function phpoot_raw($val, $var, $quiet =true)
 {
     if (is_string($val) || is_numeric($val)) {
         echo $val;
@@ -557,7 +620,7 @@ function phpoot_raw($val, $var, $error)
         echo $val[$var];
     }
     else {
-        echo ($error) ? "<!-- NON STRING DATA '$var' -->" : '';
+        echo (! $quiet) ? "<!-- NON STRING DATA '$var' -->" : '';
     }
 }
 
