@@ -1,6 +1,8 @@
 <?php
+error_reporting (E_ALL);
 require_once '../phpoot.php';
 $tmpl = new phpoot;
+$tmpl->setErrorHandling(PEAR_ERROR_PRINT);
 $tmpl->setTemplateFile('./attribute.html');
 $model_data = array(
   'table' => array(
